@@ -10,11 +10,11 @@ Language::RAM - A "Random Access Machine" Emulator
 
 =head1 VERSION
 
-Version 0.01
+Version 0.012
 
 =cut
 
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 
 
 =head1 SYNOPSIS
@@ -164,8 +164,11 @@ sub run {
   $machine->{'memory'} = {};
   $machine->{'snaps'} = {} if $_[3];
 
-  while (my ($index, $id) = each @{$machine->{'input_layout'}}) {
-    $machine->{'memory'}{$id} = $machine->{'input'}[$index] // 0;
+  foreach my $index ( 0 .. $#{$$machine{'input_layout'}}) {
+    my $id = $$machine{'input_layout'}->[$index];
+    my $input = $machine->{'input'}[$index];
+    (defined $input) or $input = 0;
+    $machine->{'memory'}{$id} = $input;
   }
 
   while($limit == -1 || $machine->{'steps'} < $limit) {
@@ -295,8 +298,11 @@ sub get_first_memory_snapshot {
     $snapshot{$$snapshots{$_}->[1]} = 0;
   }
 
-  while (my ($index, $id) = each @{$$machine{'input_layout'}}) {
-    $snapshot{$id} = $$machine{'input'}->[$index] // 0;
+  foreach my $index ( 0 .. $#{$$machine{'input_layout'}}) {
+    my $id = $$machine{'input_layout'}->[$index];
+    my $input = $$machine{'input'}->[$index];
+    (defined $input) or $input = 0;
+    $snapshot{$id} = $input;
   }
 
   return %snapshot;
@@ -870,14 +876,16 @@ Returns address of memory block if B<type> is 2.
 =cut
 
 sub eval_mem {
+  my $type = $_[2];
+  (defined $type) or $type = 0;
   unless(exists $_[1]->{'memory'}{$_[0]->[1]}) {
     $_[1]->{'memory'}{$_[0]->[1]} = 0;
   }
 
-  &inc_mem_stat($_[1], $_[0]->[1], $_[2]);
+  &inc_mem_stat($_[1], $_[0]->[1], $type);
 
-  return $_[1]->{'memory'}{$_[0]->[1]} unless ($_[2] // 0) > 0;
-  return \$_[1]->{'memory'}{$_[0]->[1]} if $_[2] == 1;
+  return $_[1]->{'memory'}{$_[0]->[1]} unless $type > 0;
+  return \$_[1]->{'memory'}{$_[0]->[1]} if $type == 1;
   return $_[0]->[1];
 }
 
@@ -1071,7 +1079,7 @@ Add read action to memory slot if B<action> is false.
 =cut
 
 sub inc_mem_stat {
-  return if ($_[2] // 0) == 2; #Bail if used in stat collection.
+  return if $_[2] == 2; #Bail if used in stat collection.
 
   unless(exists $_[0]->{'stats'}{'memory_usage'}{$_[1]}) {
     $_[0]->{'stats'}{'memory_usage'}{$_[1]} = [qw(0 0)];
